@@ -10,14 +10,16 @@
 # 2013-11-06: fixed bug in Effect.multinom() in construction of effect object
 #             when there is only one focal predictor; caused as.data.frame.effpoly() to fail
 # 2014-03-13: modified Effect.lm() to compute partial residuals. J. Fox
-
+# 2014-05-06: fixed bug in Effect.gls() when cor or var structure depends on variables in the data set. J. Fox
+# 2014-08-02: added vcov.=vcov argument to allow other methods of estimating var(coef.estimates)
 
 Effect <- function(focal.predictors, mod, ...){
     UseMethod("Effect", mod)
 }
+
 Effect.lm <- function (focal.predictors, mod, xlevels = list(), 
     default.levels = NULL, given.values,
-    se = TRUE, confidence.level = 0.95, 
+    vcov. = vcov, se = TRUE, confidence.level = 0.95, 
     transformation = list(link = family(mod)$linkfun, inverse = family(mod)$linkinv), 
     typical = mean, offset = mean, partial.residuals=FALSE, quantiles=seq(0.2, 0.8, by=0.2),
     x.var=NULL,  ...){
@@ -54,6 +56,7 @@ Effect.lm <- function (focal.predictors, mod, xlevels = list(),
     Terms <- delete.response(terms(mod))
     mf <- model.frame(Terms, predict.data, xlev = factor.levels, na.action=NULL)
     mod.matrix <- model.matrix(formula.rhs, data = mf, contrasts.arg = mod$contrasts)
+    if (is.null(x.var)) partial.residuals <- FALSE
     factors <- sapply(predict.data, is.factor)
     if (partial.residuals){
         for (predictor in focal.predictors[-x.var]){
@@ -120,22 +123,26 @@ Effect.lm <- function (focal.predictors, mod, xlevels = list(),
     } 
     if (se) {
         if (any(family(mod)$family == c("binomial", "poisson"))) {
-            dispersion <- 1
+#            dispersion <- 1
             z <- qnorm(1 - (1 - confidence.level)/2)
         }
         else {
-            dispersion <- sum(wts * (residuals(mod))^2, na.rm=TRUE)/mod$df.residual # sum(wts * mod$residuals^2)/mod$df.residual
+#            dispersion <- sum(wts * (residuals(mod))^2, na.rm=TRUE)/mod$df.residual 
             z <- qt(1 - (1 - confidence.level)/2, df = mod$df.residual)
         }
-        V2 <- dispersion * summary.lm(mod)$cov
-        V1 <- vcov(mod)
-        V <- if (inherits(mod, "fakeglm")) 
-            V1
-        else V2
-        vcov <- mod.matrix %*% V %*% t(mod.matrix)
-        rownames(vcov) <- colnames(vcov) <- NULL
-        var <- diag(vcov)
-        result$vcov <- vcov        
+# old
+#        V2 <- dispersion * summary.lm(mod)$cov
+#        V1 <- vcov(mod)
+#        V <- if (inherits(mod, "fakeglm")) 
+#            V1
+#        else V2
+# end old, begin new August 2, 2014
+        V <- vcov.(mod)  # I can see no reason to use dispersion * summary.lm(mod)$cov
+# end new
+        eff.vcov <- mod.matrix %*% V %*% t(mod.matrix)
+        rownames(eff.vcov) <- colnames(eff.vcov) <- NULL
+        var <- diag(eff.vcov)
+        result$vcov <- eff.vcov        
         result$se <- sqrt(var)
         result$lower <- effect - z * result$se
         result$upper <- effect + z * result$se
@@ -275,76 +282,144 @@ Effect.lme <- function(focal.predictors, mod, ...) {
     result
 }
 
+# Effect.gls <- function (focal.predictors, mod, xlevels = list(), default.levels = NULL, given.values,
+#     se = TRUE, confidence.level = 0.95, 
+#     transformation = NULL, 
+#     typical = mean, ...){
+#     if (missing(given.values)) 
+#         given.values <- NULL
+#     else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
+#         stop("given.values (", names(given.values[!which]), ") not in the model")
+#     formula.rhs <- formula(mod)[[3]]
+#     mod.lm <- lm(as.formula(mod$call$model), data=eval(mod$call$data))
+#     model.components <- Analyze.model(focal.predictors, mod.lm, xlevels, default.levels, formula.rhs)
+#     excluded.predictors <- model.components$excluded.predictors
+#     predict.data <- model.components$predict.data
+#     factor.levels <- model.components$factor.levels
+#     factor.cols <- model.components$factor.cols
+#     n.focal <- model.components$n.focal
+#     x <- model.components$x
+#     X.mod <- model.components$X.mod
+#     cnames <- model.components$cnames
+#     X <- model.components$X
+#     formula.rhs <- formula(mod)[c(1, 3)]
+#     nrow.X <- nrow(X)
+#     mf <- model.frame(formula.rhs, data=rbind(X[,names(predict.data),drop=FALSE], predict.data), 
+#         xlev=factor.levels)
+#     mod.matrix.all <- model.matrix(formula.rhs, data=mf, contrasts.arg=mod$contrasts)
+#     mod.matrix <- mod.matrix.all[-(1:nrow.X),]
+#     mod.matrix <- Fixup.model.matrix(mod.lm, mod.matrix, model.matrix(mod.lm), 
+#         X.mod, factor.cols, cnames, focal.predictors, excluded.predictors, typical, given.values)
+#     fit.1 <- na.omit(predict(mod))
+#     mod.2 <- lm.fit(mod.matrix.all[1:nrow.X,], fit.1)
+#     class(mod.2) <- "lm"
+#     #     assign(".y", na.omit(model.response.gls(mod)), envir=.GlobalEnv)
+#     #     assign(".X", na.omit(mod.matrix.all[1:nrow.X,]), envir=.GlobalEnv)
+#     .Data <- list(.y=na.omit(model.response.gls(mod)), .X=na.omit(mod.matrix.all[1:nrow.X,]))
+#     mod.3 <- update(mod, .y ~ .X - 1, data=.Data)
+#     #    remove(".X", ".y", envir=.GlobalEnv)
+#     discrepancy <- 100*mean(abs(fitted(mod.2)- fit.1)/(1e-10 + mean(abs(fit.1))))
+#     if (discrepancy > 1e-3) warning(paste("There is a discrepancy of", round(discrepancy, 3),
+#         "percent \n     in the 'safe' predictions used to generate effect", paste(focal.predictors, collapse="*")))
+#     effect <- mod.matrix %*% mod$coefficients
+#     result <- list(term = paste(focal.predictors, collapse="*"), 
+#         formula = formula(mod), response = response.name(mod), 
+#         variables = x, fit = effect, x = predict.data[, 1:n.focal, drop=FALSE], model.matrix = mod.matrix, data = X, 
+#         discrepancy = discrepancy, offset=0)
+#     if (se){
+#         df.res <- mod$dims[["N"]] - mod$dims[["p"]]
+#         z <- qt(1 - (1 - confidence.level)/2, df=df.res)
+#         mod.2$terms <- terms(mod)
+#         V <- vcov(mod.3)
+#         vcov <- mod.matrix %*% V %*% t(mod.matrix)
+#         rownames(vcov) <- colnames(vcov) <- NULL
+#         var <- diag(vcov)
+#         result$vcov <- vcov
+#         result$se <- sqrt(var)        
+#         result$lower <- effect - z*result$se
+#         result$upper <- effect + z*result$se
+#         result$confidence.level <- confidence.level
+#     }
+#     if (is.null(transformation$link) && is.null(transformation$inverse)){
+#         transformation$link <- I
+#         transformation$inverse <- I
+#     }
+#     result$transformation <- transformation
+#     class(result) <- "eff"
+#     result
+# }
+
 Effect.gls <- function (focal.predictors, mod, xlevels = list(), default.levels = NULL, given.values,
-    se = TRUE, confidence.level = 0.95, 
-    transformation = NULL, 
-    typical = mean, ...){
-    if (missing(given.values)) 
-        given.values <- NULL
-    else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
-        stop("given.values (", names(given.values[!which]), ") not in the model")
-    formula.rhs <- formula(mod)[[3]]
-    mod.lm <- lm(as.formula(mod$call$model), data=eval(mod$call$data))
-    model.components <- Analyze.model(focal.predictors, mod.lm, xlevels, default.levels, formula.rhs)
-    excluded.predictors <- model.components$excluded.predictors
-    predict.data <- model.components$predict.data
-    factor.levels <- model.components$factor.levels
-    factor.cols <- model.components$factor.cols
-    n.focal <- model.components$n.focal
-    x <- model.components$x
-    X.mod <- model.components$X.mod
-    cnames <- model.components$cnames
-    X <- model.components$X
-    formula.rhs <- formula(mod)[c(1, 3)]
-    nrow.X <- nrow(X)
-    mf <- model.frame(formula.rhs, data=rbind(X[,names(predict.data),drop=FALSE], predict.data), 
-        xlev=factor.levels)
-    mod.matrix.all <- model.matrix(formula.rhs, data=mf, contrasts.arg=mod$contrasts)
-    mod.matrix <- mod.matrix.all[-(1:nrow.X),]
-    mod.matrix <- Fixup.model.matrix(mod.lm, mod.matrix, model.matrix(mod.lm), 
-        X.mod, factor.cols, cnames, focal.predictors, excluded.predictors, typical, given.values)
-    fit.1 <- na.omit(predict(mod))
-    mod.2 <- lm.fit(mod.matrix.all[1:nrow.X,], fit.1)
-    class(mod.2) <- "lm"
-    #     assign(".y", na.omit(model.response.gls(mod)), envir=.GlobalEnv)
-    #     assign(".X", na.omit(mod.matrix.all[1:nrow.X,]), envir=.GlobalEnv)
-    .Data <- list(.y=na.omit(model.response.gls(mod)), .X=na.omit(mod.matrix.all[1:nrow.X,]))
-    mod.3 <- update(mod, .y ~ .X - 1, data=.Data)
-    #    remove(".X", ".y", envir=.GlobalEnv)
-    discrepancy <- 100*mean(abs(fitted(mod.2)- fit.1)/(1e-10 + mean(abs(fit.1))))
-    if (discrepancy > 1e-3) warning(paste("There is a discrepancy of", round(discrepancy, 3),
-        "percent \n     in the 'safe' predictions used to generate effect", paste(focal.predictors, collapse="*")))
-    effect <- mod.matrix %*% mod$coefficients
-    result <- list(term = paste(focal.predictors, collapse="*"), 
-        formula = formula(mod), response = response.name(mod), 
-        variables = x, fit = effect, x = predict.data[, 1:n.focal, drop=FALSE], model.matrix = mod.matrix, data = X, 
-        discrepancy = discrepancy, offset=0)
-    if (se){
-        df.res <- mod$dims[["N"]] - mod$dims[["p"]]
-        z <- qt(1 - (1 - confidence.level)/2, df=df.res)
-        mod.2$terms <- terms(mod)
-        V <- vcov(mod.3)
-        vcov <- mod.matrix %*% V %*% t(mod.matrix)
-        rownames(vcov) <- colnames(vcov) <- NULL
-        var <- diag(vcov)
-        result$vcov <- vcov
-        result$se <- sqrt(var)        
-        result$lower <- effect - z*result$se
-        result$upper <- effect + z*result$se
-        result$confidence.level <- confidence.level
-    }
-    if (is.null(transformation$link) && is.null(transformation$inverse)){
-        transformation$link <- I
-        transformation$inverse <- I
-    }
-    result$transformation <- transformation
-    class(result) <- "eff"
-    result
+                        vcov. = vcov, se = TRUE, confidence.level = 0.95, 
+                        transformation = NULL, 
+                        typical = mean, ...){
+  if (missing(given.values)) 
+    given.values <- NULL
+  else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
+    stop("given.values (", names(given.values[!which]), ") not in the model")
+  formula.rhs <- formula(mod)[[3]]
+  .data <- eval(mod$call$data)
+  mod.lm <- lm(as.formula(mod$call$model), data=.data, na.action=na.exclude)
+  model.components <- Analyze.model(focal.predictors, mod.lm, xlevels, default.levels, formula.rhs)
+  excluded.predictors <- model.components$excluded.predictors
+  predict.data <- model.components$predict.data
+  factor.levels <- model.components$factor.levels
+  factor.cols <- model.components$factor.cols
+  n.focal <- model.components$n.focal
+  x <- model.components$x
+  X.mod <- model.components$X.mod
+  cnames <- model.components$cnames
+  X <- model.components$X
+  formula.rhs <- formula(mod)[c(1, 3)]
+  nrow.X <- nrow(X)
+  mf <- model.frame(formula.rhs, data=rbind(X[,names(predict.data),drop=FALSE], predict.data), 
+                    xlev=factor.levels)
+  mod.matrix.all <- model.matrix(formula.rhs, data=mf, contrasts.arg=mod$contrasts)
+  mod.matrix <- mod.matrix.all[-(1:nrow.X),]
+  mod.matrix <- Fixup.model.matrix(mod.lm, mod.matrix, model.matrix(mod.lm), 
+                                   X.mod, factor.cols, cnames, focal.predictors, excluded.predictors, typical, given.values)
+  fit.1 <- na.omit(predict(mod))
+  mod.2 <- lm.fit(mod.matrix.all[1:nrow.X,], fit.1)
+  class(mod.2) <- "lm"
+  use <- !is.na(residuals(mod.lm))
+  .data <- .data[use, ]
+  .data$.y <- model.response.gls(mod)
+  .data$.X <- mod.matrix.all[1:nrow.X, ]
+  mod.3 <- update(mod, .y ~ .X - 1, data=.data)
+  discrepancy <- 100*mean(abs(fitted(mod.2)- fit.1)/(1e-10 + mean(abs(fit.1))))
+  if (discrepancy > 1e-3) warning(paste("There is a discrepancy of", round(discrepancy, 3),
+                                        "percent \n     in the 'safe' predictions used to generate effect", paste(focal.predictors, collapse="*")))
+  effect <- mod.matrix %*% mod$coefficients
+  result <- list(term = paste(focal.predictors, collapse="*"), 
+                 formula = formula(mod), response = response.name(mod), 
+                 variables = x, fit = effect, x = predict.data[, 1:n.focal, drop=FALSE], model.matrix = mod.matrix, data = X, 
+                 discrepancy = discrepancy, offset=0)
+  if (se){
+    df.res <- mod$dims[["N"]] - mod$dims[["p"]]
+    z <- qt(1 - (1 - confidence.level)/2, df=df.res)
+    mod.2$terms <- terms(mod)
+    V <- vcov.(mod.3)
+    eff.vcov <- mod.matrix %*% V %*% t(mod.matrix)
+    rownames(eff.vcov) <- colnames(eff.vcov) <- NULL
+    var <- diag(eff.vcov)
+    result$vcov <- eff.vcov
+    result$se <- sqrt(var)        
+    result$lower <- effect - z*result$se
+    result$upper <- effect + z*result$se
+    result$confidence.level <- confidence.level
+  }
+  if (is.null(transformation$link) && is.null(transformation$inverse)){
+    transformation$link <- I
+    transformation$inverse <- I
+  }
+  result$transformation <- transformation
+  class(result) <- "eff"
+  result
 }
 
 Effect.multinom <- function(focal.predictors, mod, 
     confidence.level=.95, xlevels=list(), default.levels=NULL,
-    given.values, se=TRUE, typical=mean, ...){    
+    given.values, vcov. = vcov, se=TRUE, typical=mean, ...){    
     if (length(mod$lev) < 3) stop("effects for multinomial logit model only available for response levels > 2")
     if (missing(given.values)) given.values <- NULL
     else if (!all(which <- colnames(given.values) %in% names(coef(mod)))) 
@@ -369,7 +444,7 @@ Effect.multinom <- function(focal.predictors, mod,
     resp.names <- make.names(mod$lev, unique=TRUE)
     resp.names <- c(resp.names[-1], resp.names[1]) # make the last level the reference level
     B <- t(coef(mod))
-    V <- vcov(mod)
+    V <- vcov.(mod)
     m <- ncol(B) + 1
     p <- nrow(B)
     r <- p*(m - 1)	
@@ -450,7 +525,7 @@ Effect.multinom <- function(focal.predictors, mod,
 
 Effect.polr <- function(focal.predictors, mod, 
     confidence.level=.95, xlevels=list(), default.levels=NULL,
-    given.values, se=TRUE, typical=mean, latent=FALSE, ...){
+    given.values, vcov.=vcov, se=TRUE, typical=mean, latent=FALSE, ...){
     if (mod$method != "logistic") stop('method argument to polr must be "logistic"')    
     if (missing(given.values)) given.values <- NULL
     else if (!all(which <- names(given.values) %in% names(coef(mod)))) 
@@ -482,7 +557,7 @@ Effect.polr <- function(focal.predictors, mod,
         x=predict.data[, focal.predictors, drop=FALSE],
         model.matrix=X0, data=X, discrepancy=0, model="polr")
     if (latent){
-        res <- eff.latent(X0, b, vcov(mod)[1:p, 1:p], se)
+        res <- eff.latent(X0, b, vcov.(mod)[1:p, 1:p], se)
         result$fit <- res$fit
         if (se){
             result$se <- res$se
@@ -501,7 +576,7 @@ Effect.polr <- function(focal.predictors, mod,
     m <- length(alpha) + 1
     r <- m + p - 1
     indices <- c((p+1):r, 1:p)
-    V <- vcov(mod)[indices, indices]
+    V <- vcov.(mod)[indices, indices]
     for (j in 1:(m-1)){  # fix up the signs of the covariances
         V[j,] <- -V[j,]  #  for the intercepts
         V[,j] <- -V[,j]}	
@@ -543,7 +618,7 @@ Effect.polr <- function(focal.predictors, mod,
 }
 
 Effect.default <- function(focal.predictors, mod, xlevels = list(), default.levels = NULL, given.values,
-    se = TRUE, confidence.level = 0.95, 
+    vcov. = vcov, se = TRUE, confidence.level = 0.95, 
     transformation = list(link = I, inverse = I), 
     typical = mean, offset = mean, ...){
     if (missing(given.values)) 
@@ -598,11 +673,11 @@ Effect.default <- function(focal.predictors, mod, xlevels = list(), default.leve
     } 
     if (se) {
         z <- qnorm(1 - (1 - confidence.level)/2)
-        V <- vcov(mod)
-        vcov <- mod.matrix %*% V %*% t(mod.matrix)
-        rownames(vcov) <- colnames(vcov) <- NULL
-        var <- diag(vcov)
-        result$vcov <- vcov    	
+        V <- vcov.(mod)
+        eff.vcov <- mod.matrix %*% V %*% t(mod.matrix)
+        rownames(eff.vcov) <- colnames(eff.vcov) <- NULL
+        var <- diag(eff.vcov)
+        result$vcov <- eff.vcov    	
         result$se <- sqrt(var)
         result$lower <- effect - z * result$se
         result$upper <- effect + z * result$se
